@@ -134,7 +134,6 @@ qw(
 qw(
 	basename
 	dirname
-	mkdirs
 	install_file
 	install_prog
 	install_lib
@@ -489,35 +488,6 @@ sub print_and_doit_noerror {
 	goto \&_doit;
 }
 
-sub _post_fork_setup_and_exec {
-	my ($close_stdin, $options, @cmd) = @_;
-	if (defined($options)) {
-		if (defined(my $dir = $options->{chdir})) {
-			if ($dir ne '.') {
-				chdir($dir) or error("chdir(\"${dir}\") failed: $!");
-			}
-		}
-		if ($close_stdin) {
-			open(STDIN, '<', '/dev/null') or error("redirect STDIN failed: $!");
-		}
-		if (defined(my $output = $options->{stdout})) {
-			open(STDOUT, '>', $output) or error("redirect STDOUT failed: $!");
-		}
-		if (defined(my $update_env = $options->{update_env})) {
-			while (my ($k, $v) = each(%{$update_env})) {
-				if (defined($v)) {
-					$ENV{$k} = $v;
-				} else {
-					delete($ENV{$k});
-				}
-			}
-		}
-	}
-	# Force execvp call to avoid shell.  Apparently, even exec can
-	# involve a shell if you don't do this.
-	exec { $cmd[0] } @cmd or error('exec (for cmd: ' . escape_shell(@cmd) . ") failed: $!");
-}
-
 sub _doit {
 	my (@cmd) = @_;
 	my $options = ref($cmd[0]) ? shift(@cmd) : undef;
@@ -531,7 +501,29 @@ sub _doit {
 	return 1 if $dh{NO_ACT};
 	my $pid = fork() // error("fork(): $!");
 	if (not $pid) {
-		_post_fork_setup_and_exec(1, $options, @cmd) // error("Assertion error: sub should not return!");
+		if (defined($options)) {
+			if (defined(my $dir = $options->{chdir})) {
+				if ($dir ne '.') {
+					chdir($dir) or error("chdir(\"${dir}\") failed: $!");
+				}
+			}
+			open(STDIN, '<', '/dev/null') or error("redirect STDIN failed: $!");
+			if (defined(my $output = $options->{stdout})) {
+				open(STDOUT, '>', $output) or error("redirect STDOUT failed: $!");
+			}
+			if (defined(my $update_env = $options->{update_env})) {
+				while (my ($k, $v) = each(%{$update_env})) {
+					if (defined($v)) {
+						$ENV{$k} = $v;
+					} else {
+						delete($ENV{$k});
+					}
+				}
+			}
+		}
+		# Force execvp call to avoid shell.  Apparently, even exec can
+		# involve a shell if you don't do this.
+		exec { $cmd[0] } @cmd;
 	}
 	return waitpid($pid, 0) == $pid && $? == 0;
 }
@@ -570,12 +562,8 @@ sub _format_cmdline {
 
 sub qx_cmd {
 	my (@cmd) = @_;
-	my $options = ref($cmd[0]) ? shift(@cmd) : undef;
 	my ($output, @output);
-	my $pid = open(my $fd, '-|') // error('fork (for cmd: ' . escape_shell(@cmd) . ") failed: $!");
-	if ($pid == 0) {
-		_post_fork_setup_and_exec(0, $options, @cmd) // error("Assertion error: sub should not return!");
-	}
+	open(my $fd, '-|', @cmd) or error('fork+exec (' . escape_shell(@cmd) . "): $!");
 	if (wantarray) {
 		@output = <$fd>;
 	} else {
@@ -669,42 +657,28 @@ sub error_exitcode {
 	}
 }
 
-
-sub _mkdirs {
-	my ($log, @dirs) = @_;
-	return if not @dirs;
-	if ($log && $dh{VERBOSE}) {
-		verbose_print(sprintf('install -m0755 -d %s', escape_shell(@dirs)));
-	}
-	return 1 if $dh{NO_ACT};
+sub install_dir {
+	my @to_create = grep { not -d $_ } @_;
+	return if not @to_create;
 	state $_loaded;
 	if (not $_loaded) {
 		$_loaded++;
 		require File::Path;
 	}
-	my %opts = (
-		# install -d uses 0755 (no umask), make_path uses 0777 (& umask) by default.
-		# Since we claim to run install -d, then ensure the mode is correct.
-		'chmod' => 0755,
-	);
+	verbose_print(sprintf('install -d %s', escape_shell(@to_create)))
+		if $dh{VERBOSE};
+	return 1 if $dh{NO_ACT};
 	eval {
-		File::Path::make_path(@dirs, \%opts);
+		File::Path::make_path(@to_create, {
+			# install -d uses 0755 (no umask), make_path uses 0777 (& umask) by default.
+			# Since we claim to run install -d, then ensure the mode is correct.
+			'chmod' => 0755,
+		});
 	};
 	if (my $err = "$@") {
 		$err =~ s/\s+at\s+\S+\s+line\s+\d+\.?\n//;
 		error($err);
 	}
-	return;
-}
-
-sub mkdirs {
-	my @to_create = grep { not -d $_ } @_;
-	return _mkdirs(0, @to_create);
-}
-
-sub install_dir {
-	my @dirs = @_;
-	return _mkdirs(1, @dirs);
 }
 
 sub rename_path {
@@ -1258,7 +1232,8 @@ sub autoscript {
 		}
 	}
 
-	if (-e $outfile && ($script eq 'postrm' || $script eq 'prerm')) {
+	if (-e $outfile && ($script eq 'postrm' || $script eq 'prerm')
+	   && !compat(5)) {
 		# Add fragments to top so they run in reverse order when removing.
 		if (not defined($sed) or ref($sed)) {
 			verbose_print("[META] Prepend autosnippet \"$filename\" to $script [${outfile}.new]");
@@ -1377,7 +1352,7 @@ sub generated_file {
 	my $dir = "debian/.debhelper/generated/${package}";
 	my $path = "${dir}/${filename}";
 	$mkdirs //= 1;
-	mkdirs($dir) if $mkdirs;
+	install_dir($dir) if $mkdirs;
 	return $path;
 }
 
@@ -1432,19 +1407,6 @@ sub addsubstvar {
 	my $substvarfile = "debian/${ext}substvars";
 	my $str = $deppackage;
 	$str .= " ($verinfo)" if defined $verinfo && length $verinfo;
-
-	if (not defined($deppackage) and not $remove) {
-		error("Bug in helper: Must provide a value for addsubstvar (or set the remove flag, but then use delsubstvar instead)")
-	}
-
-	if (defined($str) and $str =~ m/[\n]/) {
-		$str =~ s/\n/\\n/g;
-		# Per #1026014
-		warning('Unescaped newlines in the value of a substvars can cause broken substvars files (see #1025714).');
-		warning("Hint: If you really need a newline character, provide it as \"\${Newline}\".");
-		error("Bug in helper: The substvar must not contain a raw newline character (${substvar}=${str})");
-	}
-
 	my $update_logic = sub {
 		my ($line) = @_;
 		return $line if $line !~ m/^\Q${substvar}\E([?]?=)(.*)/;
@@ -2604,7 +2566,7 @@ sub setup_home_and_xdg_dirs {
 		XDG_DATA_DIRS
 		XDG_RUNTIME_DIR
 	);
-	mkdirs(@paths);
+	install_dir(@paths);
 	for my $envname (@clear_env) {
 		delete($ENV{$envname});
 	}
@@ -2671,7 +2633,7 @@ sub get_buildoption {
 		} elsif ($opt =~ m/^dherroron=(\S*)$/ && $wanted eq 'dherroron') {
 			my $value = $1;
 			if ($value ne 'obsolete-compat-levels') {
-				warning("Unknown value \"${value}\" as parameter for \"dherroron\" seen in DEB_BUILD_OPTIONS");
+				warning("Unknown value \"${value}\" as parameter for \"dherrron\" seen in DEB_BUILD_OPTIONS");
 			}
 			return $value;
 		} elsif ($opt eq $wanted) {
@@ -2720,14 +2682,15 @@ sub _executable_dh_config_file_failed {
 # the package.  Under compat 9+ it may execute the file and use its
 # output instead.
 #
-# install_dh_config_file(SOURCE, TARGET)
+# install_dh_config_file(SOURCE, TARGET[, MODE])
 sub install_dh_config_file {
-	my ($source, $target) = @_;
+	my ($source, $target, $mode) = @_;
+	$mode = 0644 if not defined($mode);
 
 	if (!compat(8) and -x $source) {
 		my @sstat = stat(_) || error("cannot stat $source: $!");
 		open(my $tfd, '>', $target) || error("cannot open $target: $!");
-		chmod(0644, $tfd) || error("cannot chmod $target: $!");
+		chmod($mode, $tfd) || error("cannot chmod $target: $!");
 		open(my $sfd, '-|', $source) || error("cannot run $source: $!");
 		while (my $line = <$sfd>) {
 			print ${tfd} $line;
@@ -2739,7 +2702,7 @@ sub install_dh_config_file {
 		# Set the mtime (and atime) to ensure reproducibility.
 		utime($sstat[9], $sstat[9], $target);
 	} else {
-		install_file($source, $target);
+		_install_file_to_path($mode, $source, $target);
 	}
 	return 1;
 }
@@ -2749,7 +2712,7 @@ sub restore_file_on_clean {
 	my $bucket_index = 'debian/.debhelper/bucket/index';
 	my $bucket_dir = 'debian/.debhelper/bucket/files';
 	my $checksum;
-	mkdirs($bucket_dir);
+	install_dir($bucket_dir);
 	if ($file =~ m{^/}) {
 		error("restore_file_on_clean requires a path relative to the package dir");
 	}
